@@ -24,9 +24,9 @@ es_GET <- function(path, index=NULL, type=NULL, metric=NULL, node=NULL,
   
   args <- ec(list(...))
   if (length(args) == 0) args <- NULL
-  tt <- GET(url, query = args, mc(make_up(), callopts))
+  tt <- GET(url, query = args, c(es_env$headers, mc(make_up(), callopts)))
   geterror(tt)
-  res <- content(tt, as = "text")
+  res <- cont_utf8(tt)
   if (!is.null(clazz)) { 
     class(res) <- clazz
     if (raw) res else es_parse(res)
@@ -51,9 +51,9 @@ index_GET <- function(index, features, raw, ...) {
   url <- paste0(url, "/", paste0(esc(index), collapse = ","))
   if (!is.null(features)) features <- paste0(paste0("_", features), collapse = ",")
   if (!is.null(features)) url <- paste0(url, "/", features)
-  tt <- GET(url, make_up(), ...)
+  tt <- GET(url, make_up(), es_env$headers, ...)
   if (tt$status_code > 202) geterror(tt)
-  jsonlite::fromJSON(content(tt, as = "text"), FALSE)
+  jsonlite::fromJSON(cont_utf8(tt), FALSE)
 }
 
 es_POST <- function(path, index=NULL, type=NULL, clazz=NULL, raw, callopts, query, ...) {
@@ -74,10 +74,10 @@ es_POST <- function(path, index=NULL, type=NULL, clazz=NULL, raw, callopts, quer
   args <- check_inputs(query)
   if (length(args) == 0) args <- NULL
   
-  tt <- POST(url, body = args, mc(make_up(), callopts), encode = "json")
+  tt <- POST(url, body = args, c(es_env$headers, mc(make_up(), callopts)), encode = "json")
   geterror(tt)
   # if(tt$status_code > 202) geterror(tt)
-  res <- content(tt, as = "text")
+  res <- cont_utf8(tt)
   if (!is.null(clazz)) { 
     class(res) <- clazz
     if (raw) res else es_parse(input = res)
@@ -88,27 +88,24 @@ es_POST <- function(path, index=NULL, type=NULL, clazz=NULL, raw, callopts, quer
 
 es_DELETE <- function(url, query = NULL, ...) {
   checkconn()
-  tt <- DELETE(url, query = query, c(make_up(), ...))
+  tt <- DELETE(url, query = query, c(make_up(), es_env$headers, ...))
   geterror(tt)
-  # if(tt$status_code > 202) stop(content(tt)$error)
-  jsonlite::fromJSON(content(tt, "text"), FALSE)
+  jsonlite::fromJSON(cont_utf8(tt), FALSE)
 }
 
 es_PUT <- function(url, body = list(), ...) {
   checkconn()
   body <- check_inputs(body)
-  tt <- PUT(url, body = body, encode = 'json', c(make_up(), ...))
+  tt <- PUT(url, body = body, encode = 'json', c(make_up(), es_env$headers, ...))
   geterror(tt)
-  # if (tt$status_code > 202) stop(content(tt)$error)
-  jsonlite::fromJSON(content(tt, "text"), FALSE)
+  jsonlite::fromJSON(cont_utf8(tt), FALSE)
 }
 
 es_GET_ <- function(url, query = NULL, ...) {
   checkconn()
-  tt <- GET(url, query = query, make_up(), ...)
+  tt <- GET(url, query = query, make_up(), es_env$headers, ...)
   geterror(tt)
-  # if(tt$status_code > 202) stop(content(tt)$error)
-  jsonlite::fromJSON(content(tt, "text"), FALSE)
+  jsonlite::fromJSON(cont_utf8(tt), FALSE)
 }
 
 check_inputs <- function(x) {
@@ -132,9 +129,12 @@ geterror <- function(z) {
   if (!is(z, "response")) stop("Input to error parser must be a httr response object")
   if (z$status_code > 202) {
     if (is.null(z$headers$statusmessage)) {
-      err <- tryCatch(content(z, "text"), error = function(e) e)
-      err <- if (is(err, "simpleError")) content(z) else err
+      err <- tryCatch(cont_utf8(z), error = function(e) e)
+      err <- if (is(err, "simpleError")) jsonlite::fromJSON(cont_utf8(z), FALSE) else err
       if (!is(err, "simpleError")) {
+        if (nchar(cont_utf8(z)) == 0) {
+          stop(http_status(z)$message, call. = FALSE)
+        }
         err <- jsonlite::fromJSON(err, simplifyVector = FALSE, simplifyDataFrame = FALSE)
         erropt <- Sys.getenv("ELASTIC_RCLIENT_ERRORS")
         if (erropt == "complete") {
@@ -147,11 +147,6 @@ geterror <- function(z) {
             msg <- tryCatch(err$error, error = function(e) e)
             if (is(msg, "simpleError") || is.null(msg)) {
               msg <- httr::http_status(z)$message
-              # if (!err$found) {
-              #   msg <- "not found"
-              # } else {
-              #   msg <- "error"
-              # }
             }
           }
           stop(z$status_code, " - ", msg, call. = FALSE)
