@@ -28,8 +28,8 @@ msearch <- function(x, raw = FALSE, asdf = FALSE, ...) {
   if (!file.exists(x)) stop("file ", x, " does not exist", call. = FALSE)
   conn <- es_get_auth()
   url <- paste0(conn$base, ":", conn$port, '/_msearch')
-  tt <- POST(url, make_up(), ..., body = upload_file(x, type = "application/json"), encode = "json")
+  tt <- POST(url, make_up(), es_env$headers, ..., body = upload_file(x, type = "application/json"), encode = "json")
   geterror(tt)
-  res <- content(tt, as = "text")
+  res <- cont_utf8(tt)
   if (raw) res else jsonlite::fromJSON(res, asdf)
 }
